/* -*- Mode: C; c-file-style: "stroustrup"; -*- */
#ifndef __NXP_CANVAS_H__
#define __NXP_CANVAS_H__

#include <gtk/gtk.h>
#include "nxplot.h"


#define NXP_CANVAS(object)            (G_TYPE_CHECK_INSTANCE_CAST ((object), nxp_canvas_get_type(), NxpCanvas))
#define NXP_CANVAS_CLASS(klass)	      (G_TYPE_CHECK_CLASS_CAST ((klass), nxp_canvas_get_type(), NxpCanvasClass))
#define NXP_IS_CANVAS(object)         (G_TYPE_CHECK_INSTANCE_TYPE ((object), nxp_canvas_get_type()))
#define NXP_IS_CANVAS_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE ((klass), nxp_canvas_get_type()))
#define NXP_CANVAS_GET_CLASS(object)  (G_TYPE_INSTANCE_GET_CLASS ((object), nxp_canvas_get_type(), NxpCanvasClass))
#define NXP_CANVAS_TYPE(object)       (G_TYPE_FROM_INSTANCE (object))
#define NXP_CANVAS_TYPE_NAME(object)  (g_type_name (NXP_CANVAS_TYPE (object)))
#define NXP_CANVAS_CLASS_TYPE(klass)  (G_TYPE_FROM_CLASS (klass))
#define NXP_CANVAS_CLASS_NAME(klass)  (g_type_name (NXP_CANVAS_CLASS_TYPE (klass)))
#define NXP_TYPE_CANVAS    	      (nxp_canvas_get_type())


typedef struct _NxpCanvas NxpCanvas;
typedef struct _NxpCanvasContainer NxpCanvasContainer;

typedef struct _NxpCanvasClass NxpCanvasClass;


#include "nxpcanvasobject.h"
#include "nxpcanvasoverlay.h"
/* #include "nxpcanvascontainer.h" */


struct _NxpCanvas
{
    GtkDrawingArea      darea;
    Nxplot             *plotter;
    NxplotRect          view;
    GSList             *objects;
    GSList             *drawable_objects;
    NxpCanvasContainer *root;
    GSList             *overlays;
    GdkGC              *xor_gc;
    gulong              update_sigid;
    guint               overlay_idle_id;
    guint               update_idle_id;
    GdkCursor          *current_cursor;
};


struct _NxpCanvasClass
{
    GtkDrawingAreaClass base;
};



GType      nxp_canvas_get_type                        (void);
GtkWidget* nxp_canvas_new                             (void);
void       nxp_canvas_set_plotter                     (NxpCanvas          *self,
						       Nxplot             *plotter);
void       nxp_canvas_draw_to                         (NxpCanvas          *self,
						       Nxplot             *plotter);
void       nxp_canvas_set_root                        (NxpCanvas          *self,
						       NxpCanvasContainer *root_container);
void       nxp_canvas_set_viewport                    (NxpCanvas          *self,
						       double              x1,
						       double              y1,
						       double              x2,
						       double              y2);
void       nxp_canvas_add_object                      (NxpCanvas          *self,
						       NxpCanvasObject    *object);
void       nxp_canvas_remove_object                   (NxpCanvas          *self,
						       NxpCanvasObject    *object);



/* overlay management */
void       nxp_canvas_add_overlay                     (NxpCanvas          *self,
						       NxpCanvasOverlay   *overlay);
void       nxp_canvas_remove_overlay                  (NxpCanvas          *self,
						       NxpCanvasOverlay   *overlay);
void       nxp_canvas_draw_overlay                    (NxpCanvas          *self,
						       NxpCanvasOverlay   *overlay);



/*  */
void       nxp_canvas_redraw_area                     (NxpCanvas          *self,
						       NxplotRect         *area);
void       nxp_canvas_set_cursor                      (NxpCanvas          *self,
						       GdkCursor          *cur);
void       nxp_canvas_get_pointer                     (NxpCanvas          *self,
						       gint               *x,
						       gint               *y,
						       GdkModifierType    *state);
void       nxp_canvas_queue_update                    (NxpCanvas          *self);
void       nxp_canvas_recompute_drawable_objects_list (NxpCanvas          *self);




__inline__ gboolean gdk_rectangle_point_in(const GdkRectangle *rect, gint x, gint y);


#endif /* __NXP_CANVAS_H__ */
