/* -*- Mode: C; c-file-style: "bsd"; -*- */
#include "nxpcanvasbbox.h"
#include <glib.h>
#include <string.h>

/*  static guint signals[LAST_SIGNAL] = { 0 }; */
/*  static NxpCanvasBBoxClass *nxp_canvas_bbox_class = NULL; */
static gpointer parent_class = NULL;

enum {
    PROP_0,
    PROP_LINE_STYLE,
    PROP_HANDLES
};


static void nxp_canvas_bbox_init            (NxpCanvasBBox      *instance,
					     gpointer            g_class);
static void nxp_canvas_bbox_class_base_init (NxpCanvasBBoxClass *class);
static void nxp_canvas_bbox_class_init      (NxpCanvasBBoxClass *class,
					     gpointer            data);
static void nxp_canvas_bbox_shutdown        (GObject            *object);
static void nxp_canvas_bbox_draw            (NxpCanvasOverlay   *self,
					     GdkDrawable        *drawable,
					     GdkGC              *gc,
					     guint               t);
static void get_property                    (GObject            *object,
					     guint               prop_id,
					     GValue             *value,
					     GParamSpec         *pspec);
static void set_property                    (GObject            *object,
					     guint               prop_id,
					     const GValue       *value,
					     GParamSpec         *pspec);




static GTypeInfo typeinfo = 
{
    /* interface types, classed types, instantiated types */
    sizeof(NxpCanvasBBoxClass), 

    (GBaseInitFunc)nxp_canvas_bbox_class_base_init, 
    NULL, // GBaseFinalizeFunc      base_finalize;

    /* classed types, instantiated types */
    (GClassInitFunc)nxp_canvas_bbox_class_init, 
    NULL, // GClassFinalizeFunc     class_finalize;
    NULL, // gconstpointer          class_data;

    /* instantiated types */
    sizeof(NxpCanvasBBox),		//guint16                instance_size;
    0,					//guint16                n_preallocs;
    (GInstanceInitFunc)nxp_canvas_bbox_init, //GInstanceInitFunc      instance_init;

    /* value handling */
    NULL,				//const GTypeValueTable	*value_table;
};


GType nxp_canvas_bbox_get_type(void)
{
    static GType type = 0;
    if (!type) 
    {
	type = g_type_register_static(NXP_TYPE_CANVAS_OVERLAY, "NxpCanvasBBox",
				      &typeinfo, 0);
    }
    return type;
}


static void nxp_canvas_bbox_init(NxpCanvasBBox *self, gpointer g_class)
{
/*      if (!nxp_canvas_bbox_class) */
/*  	nxp_canvas_bbox_class = NXP_CANVAS_BBOX_GET_CLASS(self); */
    NxpCanvasOverlay *overlay = NXP_CANVAS_OVERLAY(self);
    overlay->bbox.x = 0;
    overlay->bbox.y = 0;
    overlay->bbox.width = 0;
    overlay->bbox.height = 0;
    self->handle_size = 6;
}


static void nxp_canvas_bbox_class_base_init(NxpCanvasBBoxClass *class)
{
    /* empty */
}


static void nxp_canvas_bbox_class_init(NxpCanvasBBoxClass *class, gpointer data)
{
    GObjectClass *object_class = G_OBJECT_CLASS(class);
    NxpCanvasOverlayClass *overlay_class = NXP_CANVAS_OVERLAY_CLASS(class);

    parent_class = g_type_class_peek_parent (class);
    object_class->set_property = set_property;
    object_class->get_property = get_property;
    object_class->finalize = nxp_canvas_bbox_shutdown;
    overlay_class->draw = nxp_canvas_bbox_draw;

    g_object_class_install_property(object_class,
				    PROP_LINE_STYLE,
				    g_param_spec_int(
					"line-style", "GdkLineStyle",
					"Style of line to draw the rectangle",
					GDK_LINE_SOLID,	/* minimum */
					GDK_LINE_DOUBLE_DASH, /* maximum */
					GDK_LINE_SOLID,	/* default */
					G_PARAM_READABLE|G_PARAM_WRITABLE|
					G_PARAM_CONSTRUCT));
    g_object_class_install_property(object_class,
				    PROP_HANDLES,
				    g_param_spec_int(
					"handles",
					"NxpCanvasObjectInteractionMask",
					"Bit mask specifying which"
					" handles to draw",
					0, 0xffffff, 0xfff,
					G_PARAM_READABLE|G_PARAM_WRITABLE|
					G_PARAM_CONSTRUCT));
}


static void nxp_canvas_bbox_shutdown(GObject *object)
{
    NxpCanvasBBox *self;
    self = NXP_CANVAS_BBOX(object);
    if (G_OBJECT_CLASS(parent_class)->finalize)
	G_OBJECT_CLASS(parent_class)->finalize(object);
}


static void nxp_canvas_bbox_draw(NxpCanvasOverlay *overlay, GdkDrawable *drawable,
				 GdkGC *gc, guint t)
{
    NxpCanvasBBox *self = NXP_CANVAS_BBOX(overlay);
    g_return_if_fail(self);
    g_return_if_fail(NXP_IS_CANVAS_BBOX(self));
    gdk_gc_set_line_attributes(gc, 0, self->line_style, GDK_CAP_NOT_LAST,
			       GDK_JOIN_MITER);
    gdk_draw_rectangle(drawable, gc, FALSE, overlay->bbox.x, overlay->bbox.y,
		       overlay->bbox.width - 1, overlay->bbox.height - 1);

#define DRAW_HANDLE(h)							\
    gdk_draw_rectangle(drawable, gc, TRUE, self->handles.h.x,		\
		       self->handles.h.y, self->handles.h.width,	\
		       self->handles.h.height)
    if (self->handles.mask&NXP_COBJ_INTERACTION_RESIZE_NW_MASK)
	DRAW_HANDLE(nw);
    if (self->handles.mask&NXP_COBJ_INTERACTION_RESIZE_N_MASK)
	DRAW_HANDLE(n);
    if (self->handles.mask&NXP_COBJ_INTERACTION_RESIZE_NE_MASK)
	DRAW_HANDLE(ne);
    if (self->handles.mask&NXP_COBJ_INTERACTION_RESIZE_E_MASK)
	DRAW_HANDLE(e);
    if (self->handles.mask&NXP_COBJ_INTERACTION_RESIZE_SE_MASK)
	DRAW_HANDLE(se);
    if (self->handles.mask&NXP_COBJ_INTERACTION_RESIZE_S_MASK)
	DRAW_HANDLE(s);
    if (self->handles.mask&NXP_COBJ_INTERACTION_RESIZE_SW_MASK)
	DRAW_HANDLE(sw);
    if (self->handles.mask&NXP_COBJ_INTERACTION_RESIZE_W_MASK)
	DRAW_HANDLE(w);
}



NxpCanvasOverlay* nxp_canvas_bbox_new(gint x, gint y, gint width, gint height)
{
    GTypeInstance *instance;
    NxpCanvasBBox *self;
    NxpCanvasOverlay *overlay;

    instance = g_type_create_instance(nxp_canvas_bbox_get_type());
    self = NXP_CANVAS_BBOX(instance);
    overlay = NXP_CANVAS_OVERLAY(instance);
    overlay->bbox.x = x;
    overlay->bbox.y = y;
    overlay->bbox.width = width;
    overlay->bbox.height = height;

    return NXP_CANVAS_OVERLAY(self);
}


/**
 * nxp_canvas_bbox_set_geometry:
 * @self: 
 * @geometry: 
 * 
 * Set new bbox coordinates.
 **/
void nxp_canvas_bbox_set_geometry(NxpCanvasBBox *self, GdkRectangle *geometry)
{
    NxpCanvasOverlay *overlay;
    gboolean visible;
    gint x1, xm, x2, y1, ym, y2;

    g_return_if_fail(self);
    g_return_if_fail(NXP_IS_CANVAS_BBOX(self));
    overlay = NXP_CANVAS_OVERLAY(self);
    /* Make sure it is hidden so that we can modify as much as we want
       without interfering with drawing */
    visible = overlay->visible;
    if (visible) nxp_canvas_overlay_hide(overlay);
    memcpy(&overlay->bbox, geometry, sizeof(*geometry));

    x1 = geometry->x;
    x2 = x1 + geometry->width;
    xm = (x1 + x2) / 2;
    y1 = geometry->y;
    y2 = y1 + geometry->height;
    ym = (y1 + y2) / 2;

    /* NW handle */
    self->handles.nw.x = x1 + 1;
    self->handles.nw.y = y1 + 1;
    self->handles.nw.width = self->handle_size - 1;
    self->handles.nw.height = self->handle_size - 1;

    /* N handle */
    self->handles.n.x = xm - self->handle_size / 2;
    self->handles.n.y = y1 + 1;
    self->handles.n.width = self->handle_size;
    self->handles.n.height = self->handle_size - 1;

    /* NE handle */
    self->handles.ne.x = x2 - self->handle_size;
    self->handles.ne.y = y1 + 1;
    self->handles.ne.width = self->handle_size - 1;
    self->handles.ne.height = self->handle_size - 1;

    /* E handle */
    self->handles.e.x = x2 - self->handle_size;
    self->handles.e.y = ym - self->handle_size/2;
    self->handles.e.width = self->handle_size - 1;
    self->handles.e.height = self->handle_size;

    /* SE handle */
    self->handles.se.x = x2 - self->handle_size;
    self->handles.se.y = y2 - self->handle_size;
    self->handles.se.width = self->handle_size - 1;
    self->handles.se.height = self->handle_size - 1;

    /* S handle */
    self->handles.s.x = xm - self->handle_size/2;
    self->handles.s.y = y2 - self->handle_size;
    self->handles.s.width = self->handle_size;
    self->handles.s.height = self->handle_size - 1;

    /* SW handle */
    self->handles.sw.x = x1 + 1;
    self->handles.sw.y = y2 - self->handle_size;
    self->handles.sw.width = self->handle_size - 1;
    self->handles.sw.height = self->handle_size - 1;

    /* W handle */
    self->handles.w.x = x1 + 1;
    self->handles.w.y = ym - self->handle_size/2;
    self->handles.w.width = self->handle_size - 1;
    self->handles.w.height = self->handle_size;

    /* All changes made. We can now show it again, unless it was
       previously hidden. */
    if (visible) nxp_canvas_overlay_show(overlay);
}







static void
set_property (GObject      *object,
	      guint         prop_id,
	      const GValue *value,
	      GParamSpec   *pspec)
{
    NxpCanvasBBox *self;
    NxpCanvasOverlay *overlay;

    self = NXP_CANVAS_BBOX(object);
    overlay = NXP_CANVAS_OVERLAY(object);

    switch (prop_id)
    {

    case PROP_LINE_STYLE: {
	gboolean visible;
	if (overlay->canvas && overlay->visible) {
	    nxp_canvas_overlay_hide(overlay);
	    visible = TRUE;
	} else visible = FALSE;
	self->line_style = g_value_get_int(value);
	if (visible)
	    nxp_canvas_overlay_show(overlay);
	break;
    }
    case PROP_HANDLES: {
	gboolean visible;
	if (overlay->canvas && overlay->visible) {
	    nxp_canvas_overlay_hide(overlay);
	    visible = TRUE;
	} else visible = FALSE;
	self->handles.mask = g_value_get_int(value);
	if (visible)
	    nxp_canvas_overlay_show(overlay);
	break;
    }

	
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }

}



static void
get_property (GObject      *object,
                           guint         prop_id,
                           GValue       *value,
                           GParamSpec   *pspec)
{
    NxpCanvasBBox *self;

    self = NXP_CANVAS_BBOX(object);

    switch (prop_id)
    {
    case PROP_LINE_STYLE:
	g_value_set_int(value, self->line_style);
	break;
    case PROP_HANDLES:
	g_value_set_int(value, self->handles.mask);
	break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }

}

