/* -*- Mode: C; c-file-style: "stroustrup"; -*- */
#include "nxpcanvasoverlay.h"
#include "nxplot-marshalers.h"
#include <math.h>

enum {
    DRAW_SIGNAL,
    LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };
static NxpCanvasOverlayClass *nxp_canvas_overlay_class = NULL;
static gpointer parent_class = NULL;


NxpCanvasOverlay* nxp_canvas_overlay_new(void);

static void nxp_canvas_overlay_init            (NxpCanvasOverlay      *instance,
						gpointer               g_class);
static void nxp_canvas_overlay_class_base_init (NxpCanvasOverlayClass *class);
static void nxp_canvas_overlay_class_init      (NxpCanvasOverlayClass *class,
						gpointer               data);
static void nxp_canvas_overlay_shutdown        (GObject               *object);
static void get_property                       (GObject               *object,
						guint                  prop_id,
						GValue                *value,
						GParamSpec            *pspec);
static void set_property                       (GObject               *object,
						guint                  prop_id,
						const GValue          *value,
						GParamSpec            *pspec);


enum {
    PROP_0,
    PROP_CANVAS
};


static GTypeInfo typeinfo = 
{
    /* interface types, classed types, instantiated types */
    sizeof(NxpCanvasOverlayClass), 

    (GBaseInitFunc)nxp_canvas_overlay_class_base_init, 
    NULL, // GBaseFinalizeFunc      base_finalize;

    /* classed types, instantiated types */
    (GClassInitFunc)nxp_canvas_overlay_class_init, 
    NULL, // GClassFinalizeFunc     class_finalize;
    NULL, // gconstpointer          class_data;

    /* instantiated types */
    sizeof(NxpCanvasOverlay),		//guint16                instance_size;
    0,					//guint16                n_preallocs;
    (GInstanceInitFunc)nxp_canvas_overlay_init, //GInstanceInitFunc      instance_init;

    /* value handling */
    NULL,				//const GTypeValueTable	*value_table;
};


GType nxp_canvas_overlay_get_type(void)
{
    static GType type = 0;
    if (!type)
    {
	type = g_type_register_static(G_TYPE_OBJECT, "NxpCanvasOverlay",
				      &typeinfo, 0);
    }
    return type;
}


static void nxp_canvas_overlay_init(NxpCanvasOverlay *self, gpointer g_class)
{
    if (!nxp_canvas_overlay_class)
	nxp_canvas_overlay_class = NXP_CANVAS_OVERLAY_GET_CLASS(self);
}


static void nxp_canvas_overlay_class_base_init(NxpCanvasOverlayClass *class)
{
    /* empty */
}


static void nxp_canvas_overlay_class_init(NxpCanvasOverlayClass *class, gpointer data)
{
    GObjectClass *object_class = G_OBJECT_CLASS(class);

    parent_class = g_type_class_peek_parent (class);
    object_class->set_property = set_property;
    object_class->get_property = get_property;
    object_class->finalize = nxp_canvas_overlay_shutdown;
    g_object_class_install_property(object_class,
				    PROP_CANVAS,
				    g_param_spec_object(
					"canvas", "Parent NxpCanvas",
					"Object's parent NxpCanvas",
					NXP_TYPE_CANVAS,
					G_PARAM_READABLE|G_PARAM_WRITABLE|
					G_PARAM_CONSTRUCT));

    signals[DRAW_SIGNAL] = 
	g_signal_new("draw",
		     G_OBJECT_CLASS_TYPE(object_class),
		     G_SIGNAL_RUN_LAST,
		     G_STRUCT_OFFSET(NxpCanvasOverlayClass, draw),
		     NULL, NULL,
		     _nxplot_marshal_VOID__OBJECT_OBJECT_INT,
		     G_TYPE_NONE, 3, GDK_TYPE_DRAWABLE, GDK_TYPE_GC, G_TYPE_INT);

}


static void nxp_canvas_overlay_shutdown(GObject *object)
{
    NxpCanvasOverlay *self;
    self = NXP_CANVAS_OVERLAY(object);
    if (G_OBJECT_CLASS(parent_class)->finalize)
	G_OBJECT_CLASS(parent_class)->finalize(object);
}



void nxp_canvas_overlay_draw(NxpCanvasOverlay *self, GdkDrawable *drawable,
			     GdkGC *gc, guint t)
{
    g_return_if_fail(self);
    g_return_if_fail(NXP_IS_CANVAS_OVERLAY(self));
    g_signal_emit(G_OBJECT(self), signals[DRAW_SIGNAL], 0, drawable, gc, t);
}



/**
 * nxp_canvas_overlay_set_canvas:
 * @self: overlay
 * @canvas: new parent canvas
 * 
 * Set or change parent canvas
 **/
void nxp_canvas_overlay_set_canvas(NxpCanvasOverlay *self, NxpCanvas *canvas)
{
    g_return_if_fail(self);
    g_return_if_fail(NXP_IS_CANVAS_OVERLAY(self));
    g_return_if_fail(canvas);
    g_return_if_fail(NXP_IS_CANVAS(canvas));
    if (self->canvas)
	nxp_canvas_overlay_hide(self);
    self->canvas = canvas;
}


void nxp_canvas_overlay_show(NxpCanvasOverlay *self)
{
    g_return_if_fail(self);
    g_return_if_fail(NXP_IS_CANVAS_OVERLAY(self));
    if (!self->canvas) return;
    if (self->visible) return;
    nxp_canvas_add_overlay(NXP_CANVAS(self->canvas), self);
    self->visible = TRUE;
    self->dirty = FALSE;
}


void nxp_canvas_overlay_hide(NxpCanvasOverlay *self)
{
    g_return_if_fail(self);
    g_return_if_fail(NXP_IS_CANVAS_OVERLAY(self));
    if (!self->canvas) return;
    if (!self->visible) return;
    nxp_canvas_remove_overlay(NXP_CANVAS(self->canvas), self);
    self->visible = FALSE;
    self->dirty = FALSE;
}





static void
set_property (GObject      *object,
	      guint         prop_id,
	      const GValue *value,
	      GParamSpec   *pspec)
{
    NxpCanvasOverlay *self;

    self = NXP_CANVAS_OVERLAY(object);

    switch (prop_id)
    {

    case PROP_CANVAS: {
	NxpCanvas *canvas;

	g_return_if_fail(!self->canvas);
	canvas = g_value_get_object(value);
	if (canvas)
	    nxp_canvas_overlay_set_canvas(self, NXP_CANVAS(canvas));
	break;
    }

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }

}




static void
get_property (GObject      *object,
                           guint         prop_id,
                           GValue       *value,
                           GParamSpec   *pspec)
{
    NxpCanvasOverlay *self;

    self = NXP_CANVAS_OVERLAY(object);

    switch (prop_id)
    {
    case PROP_CANVAS:
	g_value_set_object(value, G_OBJECT(self->canvas));
	break;


    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }

}


void nxp_canvas_overlay_set_bbox(NxpCanvasOverlay *self,
				 gint x, gint y,
				 gint width, gint height)
{
    g_return_if_fail(NXP_IS_CANVAS_OVERLAY(self));
    self->bbox.x      = x;
    self->bbox.y      = y;
    self->bbox.width  = width;
    self->bbox.height = height;
}


/* Don't use this! This class is really abstract, but this function is
 * needed for language bindings. */
NxpCanvasOverlay* nxp_canvas_overlay_new(void)
{
    return g_object_new(NXP_TYPE_CANVAS_OVERLAY, NULL);
}


