/* -*- Mode: C; c-file-style: "stroustrup"; -*- */
#ifndef __NXPLOT_ARTFT_H__
#define __NXPLOT_ARTFT_H__

#include <stdio.h>
#include <gtk/gtk.h>
#include <libart_lgpl/libart.h>

#include "nxplot.h"
#include "nxplot-artft-font.h"


#define NXPLOT_ARTFT(object)\
	(G_TYPE_CHECK_INSTANCE_CAST ((object), nxplot_artft_get_type(), NxplotArtft))
#define NXPLOT_ARTFT_CLASS(klass)\
	(G_TYPE_CHECK_CLASS_CAST ((klass), nxplot_artft_get_type(), NxplotArtftClass))
#define IS_NXPLOT_ARTFT(object)\
	(G_TYPE_CHECK_INSTANCE_TYPE ((object), nxplot_artft_get_type()))
#define IS_NXPLOT_ARTFT_CLASS(klass)\
	(G_TYPE_CHECK_CLASS_TYPE ((klass), nxplot_artft_get_type()))
#define NXPLOT_ARTFT_GET_CLASS(object)\
  (G_TYPE_INSTANCE_GET_CLASS ((object), nxplot_artft_get_type(), NxplotArtftClass))
#define NXPLOT_ARTFT_TYPE(object)       (G_TYPE_FROM_INSTANCE (object))
#define NXPLOT_ARTFT_TYPE_NAME(object)  (g_type_name (NXPLOT_ARTFT_TYPE (object)))
#define NXPLOT_ARTFT_CLASS_TYPE(klass)  (G_TYPE_FROM_CLASS (klass))
#define NXPLOT_ARTFT_CLASS_NAME(klass)  (g_type_name (NXPLOT_ARTFT_CLASS_TYPE (klass)))

#define TYPE_NXPLOT_ARTFT		nxplot_artft_get_type()


G_BEGIN_DECLS


typedef struct _NxplotArtft 		NxplotArtft;
typedef struct _NxplotArtftClass 	NxplotArtftClass;
typedef struct _NxplotArtftState 	NxplotArtftState;


extern FT_Render_Mode nxplot_artft_render_mode;
extern guint          nxplot_artft_load_flags;



struct _NxplotArtftState
{
    double last_x, last_y;	/* last endpoint of a line command
                                   (cursor position) */
    double space2unit[6];	/* Affine transf. from space (user) to
                                   unit (0,0)-(1,1) coordinates */
    NxplotLinemode linemode;	/* Line dash style */
    guint32 foreground;		/* Foreground color */
    guint32 background;		/* Background color */
    guint32 fillcolor;		/* Fill color */
    guint16 fillopacity;	/* Fill opacity: 0=hollow, 0xffff=solid, [1-0xfffe]=alpha */
    double linewidth;		/* Line width (thickness) */
    ArtPathStrokeJoinType joinmode; /* this one is constant because I'm lazy :P */
    ArtPathStrokeCapType capmode;   /* this one is constant because I'm lazy :P */
    double textangle, fontsize;
    NxplotRect cliprect;	/* clipping rectangle */
    gboolean   clipping_on:1; 	/* is clipping activated? */
};


struct _NxplotArtft
{
    Nxplot base;
    GtkWidget *widget;		/* widget we are monitoring and drawing to */
    GtkAllocation allocation;	/* widget's allocated region */
    GdkPixmap *pixmap;		/* offscreen pixmap */
    GdkWindow *window;		/* the widget's window */
    ArtPixBuf *pixbuf;		/* pixel rgb buffer (in client memory) */
    ArtPixBuf *clipbuf;		/* clipped version of pixbuf */
    gint clip_x, clip_y;	/* clipping window pos */
    gint clip_w, clip_h;	/* clipping window size */
    GArray *vpath_array;	/* Array of ArtVpath: the vector path
                                   under construction */
    gboolean inpath:1;		/* TRUE when a path is under construction */
    double space2window[6];	/* Affine transformation from space to
                                   window coordinates */
    double window2space[6];	/* Affine transformation from window to
                                   space coordinates */
    double wlinewidth;		/* Line width in window coordinates */
    ArtVpathDash dash_style;
    double dashes[8];
    NxplotArtftState state;	/* current state */
    GQueue *saved_states;	/* Stack of saved states (elements are
                                   of type (NxplotArtftState *) ) */
    GQuark current_font_name;
    FT_Matrix font_transf;	/* Glyph transforming matrix */
    double font_transf_art[6];	/* Glyph transforming matrix in libart format */
    NxplotArtftFont *font;	/* Currently loaded font */
    gboolean using_symbol_font;
    double font_space2window[6];
    double font_window2space[6];
};



struct _NxplotArtftClass
{
    NxplotClass base;
    
    /* signals */
    /* update -- emitted when the drawing area is configured (size
       changed) or for when, by some other reason, it is time to draw
       something */
    guint16 update_signal;
    void (*update)(NxplotArtft *nxplot);
};


typedef struct _NxplotArtftGlyph NxplotArtftGlyph;
typedef struct _NxplotArtftTextLine NxplotArtftTextLine;

struct _NxplotArtftGlyph {
    NxplotArtftCachedGlyph *glyph;
    GdkPoint pos;		/* glyph offset coordinates */
    gint glyph_bitmap_left;	/* a workaround for a freetype2 bug */
};

struct _NxplotArtftTextLine {
    GMemChunk *mem_chunk;	/* memory chunk which allocates
                                   #NxplotArtftGlyph structures */
    GPtrArray *glyphs;		/* array of pointers to #NxplotArtftGlyph */
    FT_Vector  final_pos;	/* = baseline vector */
    FT_Vector  ascender;
    FT_Vector  descender;
    gint       face_height;	/* used for bitmap rendering */
    FT_BBox    cbox;		/* Union of the control boxes of all glyphs */
};



GType                nxplot_artft_get_type            (void);
Nxplot*              nxplot_artft_new                 (GtkWidget           *widget);
Nxplot*              nxplot_artft_new_widgetless      (int                  width,
						       int                  height);
void                 nxplot_artft_set_size            (NxplotArtft         *self,
						       int                  width,
						       int                  height);
void                 nxplot_artft_get_label_bbox      (NxplotArtft         *self,
						       NxplotHAlign         halign,
						       NxplotVAlign         valign,
						       const char          *text,
						       GdkRectangle        *bbox);
NxplotArtftTextLine* nxplot_artft_text_layout         (NxplotArtft         *self,
						       const gchar         *text);
void                 nxplot_artft_text_halign         (NxplotArtftTextLine *tl,
						       NxplotHAlign         halign,
						       double              *x,
						       double              *y);
void                 nxplot_artft_text_valign         (NxplotArtftTextLine *tl,
						       NxplotVAlign         valign,
						       double              *x,
						       double              *y);
void                 nxplot_artft_text_render         (NxplotArtft         *self,
						       NxplotArtftTextLine *tl,
						       gint                 dest_x,
						       gint                 dest_y,
						       guint32              foreground);
void                 nxplot_artft_text_get_bbox       (NxplotArtftTextLine *tl,
						       GdkRectangle        *bbox);
NxplotArtftTextLine* nxplot_artft_text_line_new       (void);
NxplotArtftGlyph*    nxplot_artft_text_line_add_glyph (NxplotArtftTextLine *tl);
void                 nxplot_artft_text_line_free      (NxplotArtftTextLine *tl);
gboolean             nxplot_artft_ready               (NxplotArtft         *self);
GdkPixbuf*           nxplot_artft_get_pixbuf          (NxplotArtft         *self);



G_END_DECLS

#endif /* __NXPLOT_ARTFT_H__ */

