/* -*- Mode: C; c-file-style: "stroustrup"; -*- */
#ifndef __NXPLOT_H__
#define __NXPLOT_H__

#include <glib-object.h>
#include <gdk/gdk.h>
#include "nxplottypebuiltins.h"

G_BEGIN_DECLS




#define NXPLOT(object)        (G_TYPE_CHECK_INSTANCE_CAST ((object), nxplot_get_type(), Nxplot))
#define NXPLOT_CLASS(klass)   (G_TYPE_CHECK_CLASS_CAST ((klass), nxplot_get_type(), NxplotClass))
#define IS_NXPLOT(object)         (G_TYPE_CHECK_INSTANCE_TYPE ((object), nxplot_get_type()))
#define IS_NXPLOT_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE ((klass), nxplot_get_type()))
#define NXPLOT_GET_CLASS(object)\
  (G_TYPE_INSTANCE_GET_CLASS ((object), nxplot_get_type(), NxplotClass))
#define NXPLOT_TYPE(object)       (G_TYPE_FROM_INSTANCE (object))
#define NXPLOT_TYPE_NAME(object)  (g_type_name (NXPLOT_TYPE (object)))
#define NXPLOT_CLASS_TYPE(klass)  (G_TYPE_FROM_CLASS (klass))
#define NXPLOT_CLASS_NAME(klass)  (g_type_name (NXPLOT_CLASS_TYPE (klass)))

#define TYPE_NXPLOT		(nxplot_get_type())
#define TYPE_NXPLOT_RECT        (nxplot_rect_get_type())
#define TYPE_NXPLOT_POINT       (nxplot_point_get_type())
#define TYPE_NXPLOT_LIST	(nxplot_list_get_type())


typedef GSList NxplotList;


typedef enum {
    NXPLOT_LINEMODE_SOLID,
    NXPLOT_LINEMODE_DOTTED,
    NXPLOT_LINEMODE_DOTDASHED,
    NXPLOT_LINEMODE_SHORTDASHED,
    NXPLOT_LINEMODE_LONGDASHED,
    NXPLOT_LINEMODE_DOTDOTDASHED,
    NXPLOT_LINEMODE_DOTDOTDOTDASHED,
    NXPLOT_LINEMODE_DISCONNECTED
} NxplotLinemode;

/* text horizontal alignment/justification */
typedef enum
{
    NXPLOT_HALIGN_LEFT,
    NXPLOT_HALIGN_CENTER,
    NXPLOT_HALIGN_RIGHT
} NxplotHAlign;

/* text vertical alignment/justification */
typedef enum {
    NXPLOT_VALIGN_BOTTOM,
    NXPLOT_VALIGN_BASELINE,
    NXPLOT_VALIGN_CENTER,
    NXPLOT_VALIGN_CAPLINE,
    NXPLOT_VALIGN_TOP
} NxplotVAlign;

typedef struct _Nxplot Nxplot;
typedef struct _NxplotClass NxplotClass;

typedef struct {
    gdouble x1, y1; /* (x1,y1) -> bottom-left corner */
    gdouble x2, y2; /* (x2,y2) -> top-right corner */
} NxplotRect;

typedef struct {
    gdouble x, y;
} NxplotPoint;

typedef NxplotPoint NxplotVec;


typedef struct {
    gdouble ascender_max, descender_max;
    gdouble ascender,     descender;
    double  width;
} NxplotLabelMetrics;


struct _Nxplot
{
    GObject base;
    /* read-only fields! */
    NxplotRect view;		/* mapped coordinates to each corner of the view */
    NxplotRect clip;		/* unimplemented -- clip region, in
                                   the same coordinates as view (*not*
                                   view-relative) */
    gboolean opened:1;
};



struct _NxplotClass
{
    GObjectClass base;

    /* pure virtual methods */
    void	(*open)		 (Nxplot *nxplot);
    void	(*close)	 (Nxplot *nxplot);
    void	(*endpath)	 (Nxplot *nxplot);
    void	(*erase)	 (Nxplot *nxplot);
    void	(*clearrect)	 (Nxplot *nxplot, double x1, double y1, double x2, double y2);
    void	(*line)		 (Nxplot *nxplot, double x1, double y1, double x2, double y2);
    void	(*space)	 (Nxplot *nxplot, double x1, double y1, double x2, double y2);
    void	(*linerel)	 (Nxplot *nxplot, double x1, double y1, double x2, double y2);
    void	(*move)		 (Nxplot *nxplot, double x, double y);
    void	(*moverel)	 (Nxplot *nxplot, double x, double y);
    void	(*circle)	 (Nxplot *nxplot, double xc, double yc, double r);

    void	(*linewidth)	 (Nxplot *nxplot, double width);
    void	(*pencolor)	 (Nxplot *nxplot, guint16 red, guint16 green, guint16 blue);
    void	(*bgcolor)	 (Nxplot *nxplot, guint16 red, guint16 green, guint16 blue);
    void	(*pencolorname)	 (Nxplot *nxplot, const char *name);
    void	(*linemode)	 (Nxplot *nxplot, NxplotLinemode mode);
    void	(*fillcolor)	 (Nxplot *nxplot, guint16 red, guint16 green, guint16 blue);
    void	(*fillcolorname) (Nxplot *nxplot, const char *name);
    void	(*fillopacity)	 (Nxplot *nxplot, guint16 opacity);

    void	(*fontname) 	 (Nxplot *nxplot, const char *font_name);
    void	(*listfonts)	 (Nxplot *nxplot, gchar ***fontnames);
    void	(*fontsize)	 (Nxplot *nxplot, double size);
    void	(*textangle)	 (Nxplot *nxplot, double angle);
    void	(*label)	 (Nxplot *nxplot, NxplotHAlign halign, NxplotVAlign valign,
				  const char *text);
    double	(*labelwidth)	 (Nxplot *nxplot, const char *text);
    void	(*labelmetrics)	 (Nxplot *nxplot, const char *text, NxplotLabelMetrics *metrics);
    double	(*descender)	 (Nxplot *nxplot);

    void	(*savestate)	 (Nxplot *nxplot);
    void	(*restorestate)	 (Nxplot *nxplot);
    void        (*setmatrix)     (Nxplot *nxplot, double m0, double m1,
				  double m2, double m3, double tx, double ty);
    void        (*concat)        (Nxplot *nxplot, double m0, double m1,
				  double m2, double m3, double tx, double ty);
    void	(*rotate)	 (Nxplot *nxplot, double theta);
    void	(*scale)	 (Nxplot *nxplot, double sx, double sy);
    void	(*translate)	 (Nxplot *nxplot, double tx, double ty);

    /* These are only available to screen drawable plotters */
    void	(*space2window)	 (Nxplot *nxplot, double x, double y, double *wx, double *wy);
    void	(*window2space)	 (Nxplot *nxplot, double wx, double wy, double *x, double *y);
    void	(*setcliprect)   (Nxplot *nxplot, NxplotRect *clip_rect);
};



GType        nxplot_get_type            (void) G_GNUC_CONST;
GType        nxplot_rect_get_type       (void) G_GNUC_CONST;
GType        nxplot_list_get_type       (void) G_GNUC_CONST;
NxplotRect*  nxplot_rect_new            (double              x1,
					 double              y1,
					 double              x2,
					 double              y2);
void         nxplot_rect_free           (NxplotRect         *rect);
NxplotRect*  nxplot_rect_copy           (NxplotRect         *rect);

GType        nxplot_point_get_type      (void) G_GNUC_CONST;
NxplotPoint* nxplot_point_new           (double              x,
					 double              y);
void         nxplot_point_free          (NxplotPoint        *point);
NxplotPoint* nxplot_point_copy          (NxplotPoint        *point);
gboolean     nxplot_rect_intersect      (NxplotRect         *src1,
					 NxplotRect         *src2,
					 NxplotRect         *dest);
void         nxplot_rect_space2window   (Nxplot             *pl,
					 NxplotRect         *srect,
					 GdkRectangle       *wrect);
void         nxplot_rect_window2space   (Nxplot             *pl,
					 GdkRectangle       *wrect,
					 NxplotRect         *srect);
void         nxplot_vec_space2window    (Nxplot             *pl,
					 NxplotVec          *svec,
					 NxplotVec          *wvec);
void         nxplot_label_metrics_union (NxplotLabelMetrics *result,
					 NxplotLabelMetrics *a,
					 NxplotLabelMetrics *b);

void   nxplot_open          (Nxplot               *nxplot);
void   nxplot_close         (Nxplot               *nxplot);
void   nxplot_endpath       (Nxplot               *nxplot);
void   nxplot_erase         (Nxplot               *nxplot);
void   nxplot_clearrect     (Nxplot               *nxplot,
			     double                x1,
			     double                y1,
			     double                x2,
			     double                y2);
void   nxplot_line          (Nxplot               *nxplot,
			     double                x1,
			     double                y1,
			     double                x2,
			     double                y2);
void   nxplot_space         (Nxplot               *nxplot,
			     double                x1,
			     double                y1,
			     double                x2,
			     double                y2);
void   nxplot_linerel       (Nxplot               *nxplot,
			     double                x1,
			     double                y1,
			     double                x2,
			     double                y2);
void   nxplot_linewidth     (Nxplot               *nxplot,
			     double                width);
void   nxplot_pencolor      (Nxplot               *nxplot,
			     guint16               red,
			     guint16               green,
			     guint16               blue);
void   nxplot_bgcolor       (Nxplot               *nxplot,
			     guint16               red,
			     guint16               green,
			     guint16               blue);
void   nxplot_pencolorname  (Nxplot               *nxplot,
			     const char           *name);
void   nxplot_move          (Nxplot               *nxplot,
			     double                x,
			     double                y);
void   nxplot_moverel       (Nxplot               *nxplot,
			     double                x,
			     double                y);
void   nxplot_fontname      (Nxplot               *nxplot,
			     const char           *font_name);
void   nxplot_listfonts     (Nxplot               *nxplot,
			     gchar              ***fontnames);
void   nxplot_fontsize      (Nxplot               *nxplot,
			     double                size);
void   nxplot_textangle     (Nxplot               *nxplot,
			     double                angle);
void   nxplot_linemode      (Nxplot               *nxplot,
			     NxplotLinemode        mode);
void   nxplot_fillcolor     (Nxplot               *nxplot,
			     guint16               red,
			     guint16               green,
			     guint16               blue);
void   nxplot_fillcolorname (Nxplot               *nxplot,
			     const char           *name);
void   nxplot_fillopacity   (Nxplot               *nxplot,
			     guint16               opacity);
void   nxplot_label         (Nxplot               *nxplot,
			     NxplotHAlign          halign,
			     NxplotVAlign          valign,
			     const char           *text);
double nxplot_labelwidth    (Nxplot               *nxplot,
			     const char           *text);
void   nxplot_labelmetrics  (Nxplot               *nxplot,
			     const char           *text,
			     NxplotLabelMetrics   *metrics);
double nxplot_descender     (Nxplot               *nxplot);
void   nxplot_circle        (Nxplot               *nxplot,
			     double                xc,
			     double                yc,
			     double                r);
void   nxplot_savestate     (Nxplot               *nxplot);
void   nxplot_restorestate  (Nxplot               *nxplot);
void   nxplot_setmatrix     (Nxplot               *nxplot,
			     double                m0,
			     double                m1,
			     double                m2,
			     double                m3,
			     double                tx,
			     double                ty);
void   nxplot_concat        (Nxplot               *nxplot,
			     double                m0,
			     double                m1,
			     double                m2,
			     double                m3,
			     double                tx,
			     double                ty);
void   nxplot_rotate        (Nxplot               *nxplot,
			     double                theta);
void   nxplot_scale         (Nxplot               *nxplot,
			     double                sx,
			     double                sy);
void   nxplot_translate     (Nxplot               *nxplot,
			     double                tx,
			     double                ty);
void   nxplot_space2window  (Nxplot               *nxplot,
			     double                x,
			     double                y,
			     double               *wx,
			     double               *wy);
void   nxplot_window2space  (Nxplot               *nxplot,
			     double                wx,
			     double                wy,
			     double               *x,
			     double               *y);
void   nxplot_setcliprect   (Nxplot               *nxplot,
			     NxplotRect           *clip_rect);
void   nxplot_box           (Nxplot               *nxplot,
			     double                x1,
			     double                y1,
			     double                x2,
			     double                y2);

G_END_DECLS

#endif /* __NXPLOT_H__ */

